/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *  FLINES.CPP -- line class                           V1.2  *
 *                                                           *
 *  Language:      Borland C++                               *
 *  System:        Gateway 386/25, MS-DOS 5.0                *
 *  Application:   CSE 687                                   *
 *  Author:        Jim Fawcett, CST 2-127, (315)443-4454     *
 *                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */


#include <stdlib.h>        // ANSI:     exit
#include <ctype.h>         // ANSI:     isspace
#include "flines.hpp"      // FAWCETT:  prototypes for flines


//----< fline::fline(void) >---------------------------------//

fline::fline(void) {

  ifstrm.open("con:",ios::in);
  if (!ifstrm.good()) {
    cerr << "can't open stdin\n";
    exit(1);
  }
  top_position = position = prev_position = 0;
}

/*----< fline::fline(char *filename) >-----------------------*/

fline::fline(char *filename) {

  ifstrm.open(filename,ios::in);
  if (!ifstrm.good()) {
    cerr << "can't open " << filename << "\n";
    exit(1);
  }
  prev_position = 0;
  top_position = position = ifstrm.tellg();
}

/*----< fline::~fline(void) >--------------------------------*/

fline::~fline(void) {

  ifstrm.close();
}

/*----< int next(fline &line) >------------------------------*/

int next(fline &line) {

  if (line.ifstrm.eof())
    return 0;
  else {
    line.prev_position = line.position;
    line.position = line.ifstrm.tellg();
    line.ifstrm >> line.str;
  }
  return 1;
}

//----< int prev(fline &line) >------------------------------//

int prev(fline &line) {

  if (line.prev_position == line.position)
    return 0;
  line.ifstrm.seekg(line.prev_position);
  line.ifstrm >> line.str;
  line.position = line.prev_position;
  return 1;
}

//----< int eol(fline &line) >-------------------------------//

int eol(fline &line) {

  if (index(line.str) >= length(line.str))
    return 1;
  else
    return 0;
}

/*----< int done(fline &line) >------------------------------*/

int done(fline &line) {

  if (line.ifstrm.eof())  // ios::eof() returns nonzero on eof
    return 1;
  return 0;
}

//----< void rewind(fline &line) >---------------------------//

void rewind(fline &line) {

  line.ifstrm.seekg(line.top_position);
  line.ifstrm.clear();
}

/*----< string collect_parens(fline &line) >-----------------*/

string collect_parens(fline &line) {

  string save, par_str, null_str;
  if (match(line.str, "(") != '\0') {
    save = head(line.str);
    par_str = tail(line.str);
    int j = 0;
    int indx = index(par_str);
    while((j!=0)||(par_str[indx] == '(')) {
      if (indx == length(par_str)) {
	next(line);
	par_str += eatwhite(line.str);
      }
      if (par_str[indx] == '(')
	j++;
      if (par_str[indx] == ')')
	j--;
      indx++;
    }
    par_str[indx];
    string temp = head(par_str);
  // point to end of paren in line.str
    line.str[0];
    line.str = save + tail(par_str);
    return temp;
  }
  return null_str;
}

/*----< string collect_comments(fline &line) >---------------*/

string collect_comments(fline &line) {

  string save, null_str;
  if ((match(line.str, "/*")) != '\0') {
    save = head(line.str);
    string com_str = tail(line.str);
    while(match(line.str, "*/") == '\0') {
      if(!next(line))
	return com_str;
      com_str += "\n" + line.str;
    }
    match(com_str, "*/");
    select(com_str,2);
    com_str = head(com_str);
    if (length(tail(com_str)) == 0) {
      next(line);
      line.str = save + line.str;
    }
    else
      line.str = save + tail(com_str);
    return com_str;
  }
  else if ((match(line.str, "//")) != '\0') {
    save = head(line.str);
    string com_str = tail(line.str);
    next(line);
    line.str = save + line.str;
    return com_str;
  }
  return null_str;
}

/*----< string collect_quotes(fline &line) >-----------------*/

string collect_quotes(fline &line) {

  string save, null_str;
  if ((match(line.str, "\"")) != '\0') {
    save = head(line.str);
    string quo_str = tail(line.str);
    line.str[index(line.str)] = ' ';
    while((match(line.str, "\"") == '\0') ||
	 ((char)select(line.str,-1) == '\\')) {
    // didn't find matching quote, or it was escaped, so get more
      if(!next(line))
	return quo_str;
      quo_str += "\n" + line.str;
    }
    quo_str[0] = ' ';
    match(quo_str, "\"");
    select(quo_str,1);
    line.str = save + tail(quo_str);
    quo_str = head(quo_str);
    quo_str[0] = '\"';
    return quo_str;
  }
  else if ((match(line.str, "\'")) != '\0') {
    save = head(line.str);
    string quo_str = tail(line.str);
    quo_str[0] = ' ';
    match(quo_str, "\'");
    if ((char)select(quo_str,1) != '\'')
      select(quo_str,-1);
    quo_str = head(quo_str);
    line.str = save + tail(quo_str);
    quo_str[0] = '\'';
    return quo_str;
  }
  return null_str;
}
